package models

import (
	"github.com/beego/beego/v2/client/orm"
	"time"
)

type CygxIndustrialArticleGroupManagement struct {
	Id                     int       `orm:"column(id);pk" description:"主键ID"`
	CygxArticleId          int       `description:"cygx_article表id"`
	ArticleId              int       `description:"文章ID"`
	IndustrialManagementId int       `description:"cygx_industrial_management表的主键ID"`
	Source                 int       `description:"来源，1 活动，2专项调研"`
	CreateTime             time.Time `description:"创建时间"`
}

// IndustryArticleMinMaxPublishTime 产业文章最大最小发布时间
type IndustryArticleMinMaxPublishTime struct {
	IndustrialManagementId int       `description:"产业ID"`
	MinPublishTime         time.Time `description:"文章最小发布时间"`
	MaxPublishTime         time.Time `description:"文章最大发布时间"`
}

// GetIndustryArticleMinMaxPublishTime 获取产业文章最大最小发布时间
func GetIndustryArticleMinMaxPublishTime(condition string, pars []interface{}) (list []*IndustryArticleMinMaxPublishTime, err error) {
	sql := `SELECT
				a.industrial_management_id,
				MIN(b.publish_date) AS min_publish_time,
				MAX(b.publish_date) AS max_publish_time
			FROM
				cygx_industrial_article_group_management AS a
			JOIN cygx_article AS b ON a.article_id = b.article_id
			WHERE
				1 = 1`
	if condition != `` {
		sql += condition
	}
	sql += `GROUP BY a.industrial_management_id`
	_, err = orm.NewOrm().Raw(sql, pars).QueryRows(&list)
	return
}

// GetIndustrialArticleGroupManagementList  查询产业
func GetIndustrialArticleGroupManagementList(condition string, pars []interface{}) (list []*CygxIndustrialArticleGroupManagement, err error) {
	sql := `SELECT
				*
			FROM
				cygx_industrial_article_group_management AS a
			WHERE
				1 = 1 `
	if condition != `` {
		sql += condition
	}
	_, err = orm.NewOrm().Raw(sql, pars).QueryRows(&list)
	return
}
